#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";
require "urllist.pl";


&main();
exit(0);


sub main
{
	&HTML::print_content();
	&HTML::print_header();

	my $chan = &HTML::get("chan", 1);
	my $chan_name = &DBOps::get_chan_name($chan);

	printf "<h1>" . &Lang::get("chanstat-for") . "</h1>\n", $chan_name;

	printf "<h2>" . &Lang::get("chanstat-hourly") . "</h2>\n";
	&hourly($chan);

	printf "<h2>" . &Lang::get("chanstat-weekdaily") . "</h2>\n";
	&weekdaily($chan);

	printf "<h2>" . &Lang::get("chanstat-misc") . "</h2>\n";
	&urlstat($chan);

	printf "<h2>" . &Lang::get("chanstat-domain-days") . "</h2>\n", 30;
	&domain($chan, 30, 10);

	printf "<h2>" . &Lang::get("chanstat-alltime") . "</h2>\n";
	&alltime($chan, 10);

	my $url = &URL::chan($chan);
	my $l = &Lang::get("link-chan");
	print "<p>" . &Lang::get("link-go-to") . 
			"<a href=\"$url\">\u$l</a></p>\n";

	&HTML::print_footer(&DB::process_time());
}


# 0: chan
sub urlstat
{
	my ($id) = @_;
	my ($query, $result);

	# url count
	$query = "SELECT COUNT(id) FROM urls WHERE chan = $id";
	$result = &DB::query($query);
	my ($uniq_url_count) = $result->fetchrow;

	# wanha count
	$query = "SELECT COUNT(id), SUM(count) FROM urls " .
			"WHERE count > 1 AND chan = $id";
	$result = &DB::query($query);
	my ($url_count, $wanha_count) = $result->fetchrow;

	$wanha_count -= $url_count;
	my $total_url_count = $uniq_url_count + $wanha_count;
	printf "<p>" . &Lang::get("chanstat-urls-wanha-perc") . "</p>\n",
			$total_url_count,
			$wanha_count,
			($wanha_count * 100 / $total_url_count);
}


# 0: chan
# 1: width = 12
# 2: height = 64
sub hourly
{
	my ($id, $width, $height) = @_;
	my ($query, $result);
	$width = 16 if (! defined($width));
	$height = 64 if (! defined($height));

	my $subquery = "SELECT TO_CHAR(time, 'hh24') AS hour, id FROM urls " .
			"WHERE chan = $id";
	$query = "SELECT DISTINCT hour, COUNT(id) FROM ($subquery) AS hour " .
			"GROUP BY hour ORDER BY hour";
	$result = &DB::query($query);

	my @col;
	my @data;
	my $hi = 1;

	while (my ($hour, $count) = $result->fetchrow) {
		$hour =~ s/^0//;
		while ($hour > $#data + 1) {
			push(@data, 0);
			push(@col, $#col + 1);
		}
		push(@data, $count);
		push(@col, $#col + 1);
		$hi = $count if ($count > $hi);
	}

	&print_graph($#data + 1, $width, $height, $hi, @data, @col);
}


# 0: chan
# 1: width = 12
# 2: height = 64
sub weekdaily
{
	my ($id, $width, $height) = @_;
	my ($query, $result);
	$width = 16 if (! defined($width));
	$height = 64 if (! defined($height));

	# TODO: start with <day>

	my $subquery = "SELECT TO_CHAR(time, 'd') AS day, id FROM urls " .
			"WHERE chan = $id";
	$query = "SELECT DISTINCT day, COUNT(id) FROM ($subquery) AS hour " .
			"GROUP BY day ORDER BY day";
	$result = &DB::query($query);

	my @order = (2, 3, 4, 5, 6, 7, 1);
	my @col = ( &Lang::get("short-mon"), &Lang::get("short-tue"),
			&Lang::get("short-wed"), &Lang::get("short-thu"),
			&Lang::get("short-fri"), &Lang::get("short-sat"),
			&Lang::get("short-sun") );
	my @data = ( 0, 0, 0, 0, 0, 0, 0 );
	my $hi = 1;

	while (my ($day, $count) = $result->fetchrow) {
		$data[($day - 1)] = $count;
		$hi = $count if ($count > $hi);
	}

	&print_graph(7, $width, $height, $hi, @data, @col);
}


# 0: chan
# 1: n
# (2: seek)
sub alltime
{
	my ($chan_id, $n, $seek) = @_;
	my @urls;

	@urls = &get_alltime($chan_id, $n, $seek);

	&URLList::print(0, @urls);
}


# 0: chan
# 1: n
# r: csv (url_id,url_time,url_count,nick_id,nick_name,url)
sub get_alltime
{
	my ($chan_id, $n, $seek) = @_;
	my ($query, $result);
	my @ret;

	$query = "SELECT urls.id, urls.url, " .
				&DBOps::get_date_fmt("urls.time") . ", " .
				"urls.count, nicks.nick, nicks.id " .
			"FROM urls, nicks " .
			"WHERE urls.count > 1 AND urls.chan = $chan_id " .
				"AND urls.nick = nicks.id " .
			"ORDER BY urls.count DESC, urls.time ASC " .
			"LIMIT $n";
	$query .= " OFFSET $seek" if (defined($seek));

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($url_id, $url_url, $url_time, $url_count,
				$nick_name, $nick_id) =
			$result->fetchrow) {
		$nick_name =~ s/,/./g;
		push(@ret, "$url_id,$url_time,$url_count," .
				"$nick_id,$nick_name,$url_url");
	}
	return @ret;
}


# 0: chan
# 1: days
# 2: count
sub domain
{
	my ($chan, $days, $count) = @_;
	my ($query, $result);

	$query = "SELECT url FROM urls WHERE chan = $chan AND " .
			"CURRENT_TIMESTAMP - time < INTERVAL '$days days'";
	$result = &DB::query($query);
	if ($result->ntuples == 0) {
		print "<p>" . &Lang::get("chanstat-no-urls") . "</p>\n";
		return;
	}

	my $url_count = $result->ntuples;
	my %domains;
	while (my ($url) = $result->fetchrow) {
		my ($domain) = "$url/" =~ /.*?:\/\/(.*?)\//;
		$domain =~ s/www\.//;
		$domains{$domain}++;
	}

	my (@top_domain, @top_count);
	while (my ($domain, $count) = each %domains) {
		push(@top_domain, $domain);
		push(@top_count, $count);
	}

	for (my $i = 0; $i <= $#top_domain - 1; $i++) {
		my $ti = $i;
		my $tc = $top_count[$i];
		for (my $j = $i + 1; $j <= $#top_domain; $j++) {
			if ($top_count[$j] > $tc) {
				$tc = $top_count[$j];
				$ti = $j;
			}
		}
		if ($ti != $i) {
			my $t;
			$t = $top_domain[$i];
			$top_domain[$i] = $top_domain[$ti];
			$top_domain[$ti] = $t;
			$t = $top_count[$i];
			$top_count[$i] = $top_count[$ti];
			$top_count[$ti] = $t;
		}
	}

	my $l_domain = &Lang::get("col-domain");
	my $l_count = &Lang::get("col-times-pasted");
	my $l_of_all = &Lang::get("col-share-of-all");
	print <<EOF;
<table class="domains">
	<tr>
		<th>$l_domain</th>
		<th>$l_count</th>
		<th>$l_of_all</th>
	</tr>
EOF
	my $d_sum = 0;
	for (my $i = 0; $i < $count; $i++) {
		my $p_domain = $top_domain[$i];
		my $p_count = $top_count[$i];
		my $p_perc = sprintf("%.2f", $p_count / $url_count * 100);
		my $class = $i % 2 == 1 ? " class=\"zebra\"" : "";
		$d_sum += $p_count;
		print <<EOF;
	<tr$class>
		<td>$p_domain</td>
		<td>$p_count</td>
		<td>$p_perc%</td>
	</tr>
EOF
	}
	my $l_total = &Lang::get("col-total");
	my $d_perc = sprintf("%.2f", $d_sum / $url_count * 100);
	print <<EOF;
	<tr class="sum">
		<td>$l_total</th>
		<td>$d_sum</td>
		<td>$d_perc%</td>
	</tr>
</table>
EOF
}


# 0: count
# 1: width
# 2: height
# 3: highest
# ...: data
# ...: cols
sub print_graph
{
	my ($n, $width, $height, $hi) = @_;

	print <<EOF;
<table class="graph">
	<tr valign="bottom" class="ul">
EOF
	for (my $i = 0; $i < $n; $i++) {
		my $count = $_[($i + 4)];
		my $th = int($count / $hi * $height);
		my $dim = "width=\"$width\" height=\"$th\"";
		my $class = $i == $n - 1 ? " class=\"nol\"" : "";
		print "\t\t<td$class><img src=\"blue24.png\" alt=\"\" " .
				"$dim /></td>\n";
	}
	print "\t</tr>\n\t<tr>\n";
	for (my $i = 0; $i < $n; $i++) {
		my $class = $i == $n - 1 ? " class=\"nol\"" : "";
		print "\t\t<td$class>" . $_[($i + $n + 4)] . "</td>\n";
	}
	print "\t</tr>\n</table>\n";
}
